extern int     PERIOD          =      350;
extern int     STOP_LOSS       =      54;
extern int     TAKE_PROFIT     =      0;
extern int     INSIDE_LEVEL    =      24;
extern int     TRAILING_STOP   =      35;
extern int     TRAILING_STEP   =      35;
extern int     ORDER_STEP      =      9;
int     SLIPPAGE        =      10;
extern bool    LOT_IS_FIXED    =    true;
extern double  LOT             =      0.1;
bool    LOT_CORRECTION  =   false;
extern int     MAGIC_NUMBER    =     1;
bool    PRINT_REPORT    =   false; 
int stop_loss;           
int take_profit;          
int inside_level;         
int trailing_level;       
int trailing_step;        
int order_step;           
int slippage;             
double lot;               

bool buy_open    = false; 
bool sell_open   = false; 
double CalcHigh;          
double CalcLow;           

double order_open_price;  
double spread;            
int stop_level;           
int order_type;           
int order_ticket;         
int orders_total;
double order_stop_loss;
double sl;
double tp;
double target;
double price;
string err_msg;
bool start;
bool alert;
double margin;

int init()

{
   
   start = true;
   alert = false;

   if(PERIOD < 3)
   {
      err_msg = "PERIOD = " + DoubleToStr(PERIOD,0) + " >= 3";
      start = false;
   }


   if(STOP_LOSS <= 0) 
   {
      err_msg = "STOP_LOSS = " + DoubleToStr(STOP_LOSS,0) + " > 0";
      start = false;
   }


   if(TAKE_PROFIT<0)
   {
      err_msg = "TAKE_PROFIT = " + DoubleToStr(TAKE_PROFIT,0) + " >= 0";
      start = false;
   }

   if(INSIDE_LEVEL<1)
   {
      err_msg = "INSIDE_LEVEL = " + DoubleToStr(INSIDE_LEVEL,0) + " >= 1";
      start = false;
   }

   if(TRAILING_STOP<0)
   {
      err_msg = "TRAILING_STOP = " + DoubleToStr(TRAILING_STOP,0) + " >= 0";
      start = false;
   }

   if(TRAILING_STEP < 0)
   {
      err_msg = "TRAILING_STEP = " + DoubleToStr(TRAILING_STEP,0) + " >= 0";
      start = false;
   }

   if(TRAILING_STEP > TRAILING_STOP)
   {
      err_msg = "TRAILING_STEP = " + DoubleToStr(TRAILING_STEP,0) + " < TRAILING_STOP";
      start = false;
   }

   if(ORDER_STEP<0)
   {
      err_msg = "ORDER_STEP = " + DoubleToStr(ORDER_STEP,0) + " >=0";
      start = false;
   }

   if(SLIPPAGE<0)
   {
      err_msg = "SLIPPAGE = " + DoubleToStr(SLIPPAGE,0) + " >=0";
      start = false;
   }

   if(!OrderCalcMargin(Symbol(),1,Ask,margin) && !LOT_IS_FIXED)
   {
      err_msg = " " + Symbol() +": LOT_IS_FIXED = true";
      start = false;
   }



   stop_loss      = STOP_LOSS;
   take_profit    = TAKE_PROFIT;
   inside_level   = INSIDE_LEVEL;
   trailing_level = TRAILING_STOP;
   trailing_step  = TRAILING_STEP;
   slippage       = SLIPPAGE;
   order_step     = ORDER_STEP;


   if(Digits == 3 || Digits == 5)
   {
      stop_loss      = stop_loss      * 10;
      take_profit    = take_profit    * 10;
      inside_level   = inside_level   * 10;
      trailing_level = trailing_level * 10;
      trailing_step  = trailing_step  * 10;
      slippage       = slippage       * 10;
      order_step     = order_step     * 10;
   }
   start();
   return(0);
}

int deinit()

  {
   return(0);
  }

int start()

  {
   
   if(!IsExpertEnabled() && !IsTesting())
   {
      err_msg = "\"Expert Advisors\"";
      start=false;
   } 

   if(!start)
   {
      if(!alert)
      {
         Alert("","\n",err_msg);
         Print(err_msg);
         alert=true;
      }
      return(-1);
   }


   if(iBars(Symbol(), PERIOD_M5) < PERIOD)
   {
      Print("");
      return;
   }

   else
   {

      CalcHigh = 0.0;
      CalcLow  = 0.0;
      
      int high_index = iHighest(Symbol(), PERIOD_M5, MODE_HIGH, PERIOD,0);
      int low_index = iLowest(Symbol(), PERIOD_M5, MODE_LOW, PERIOD,0);
      
      CalcHigh = iHigh(Symbol(), PERIOD_M5, high_index);
      CalcLow  =  iLow(Symbol(), PERIOD_M5, low_index);
   }


   if(CalcHigh < Point || CalcLow < Point) return;


   stop_level = MarketInfo(Symbol(),MODE_STOPLEVEL);
   spread = NormalizeDouble(Ask - Bid, Digits); 


   if(Ask <= NormalizeDouble(CalcHigh - inside_level * Point, Digits)) buy_open  = true;
   if(Bid >= NormalizeDouble(CalcLow  + inside_level * Point, Digits)) sell_open = true;


   WorkWithPositions();


   WorkWithPendidngOrders();


   if(buy_open) OpenOrderBuyStop();


   if(sell_open) OpenOrderSellStop();

   return(0);
 }
 
void WorkWithPositions()

{
   orders_total = OrdersTotal();
   for(int pos = orders_total-1; pos>=0; pos--)
   {
      if(OrderSelect(pos, SELECT_BY_POS, MODE_TRADES))
      {
         if (OrderSymbol() != Symbol() || OrderMagicNumber() != MAGIC_NUMBER) continue; 
         {
            order_open_price = OrderOpenPrice();
            order_type       = OrderType();
            order_stop_loss  = OrderStopLoss();
            order_ticket     = OrderTicket();
            
            if(order_type == OP_BUY)
            {
               buy_open = false;

               
               if(order_stop_loss == 0 || trailing_level == 0 || trailing_step == 0) continue; 
   
               target = NormalizeDouble(MathMax(order_open_price, order_stop_loss) + trailing_level*Point, Digits);
 
               if(Bid > target)
               {
                  sl = MathMax(order_open_price, order_stop_loss) + trailing_step * Point;
     
                  if(OrderModify(order_ticket, order_open_price, sl, OrderTakeProfit(), 0, Blue))
                     if(PRINT_REPORT)Print("Buy # ", order_ticket);  
                  else
                     if(PRINT_REPORT)Print("Buy #", order_ticket);
                  
                  Sleep(2000);
                  RefreshRates(); 

               }
               continue;
            }

            else if(order_type == OP_SELL)
            {
               sell_open = false;

               
               if(order_stop_loss == 0 || trailing_level == 0 || trailing_step == 0) continue;

               target = NormalizeDouble(MathMin(order_open_price, order_stop_loss) - trailing_level*Point, Digits);

               if(Ask < target)
               {
   
                  sl = MathMin(order_open_price, order_stop_loss) - trailing_step * Point;

                  if(OrderModify(order_ticket, order_open_price, sl, OrderTakeProfit(),0,Blue))
                     if(PRINT_REPORT)Print("Sell # ",order_ticket);
                  else
                     if(PRINT_REPORT)Print("Sell #",order_ticket);
                     
                  Sleep(2000);
                  RefreshRates(); 
               }

            }
            
         }

      }              

   }
}

void WorkWithPendidngOrders()

{
   
   for(int pos=0; pos < OrdersTotal(); pos++)
   {
      if(OrderSelect(pos,SELECT_BY_POS,MODE_TRADES))
      {

         if( OrderSymbol() != Symbol() || OrderMagicNumber() != MAGIC_NUMBER) continue;
         
         order_ticket = OrderTicket();
         order_type = OrderType();
         order_open_price = OrderOpenPrice();
         
         if(order_type == OP_BUYSTOP)
         {
            buy_open = false;
            
         
            if(( NormalizeDouble(CalcHigh - order_step * Point + spread, Digits) < NormalizeDouble(order_open_price - order_step * Point + spread, Digits)) && 
               ( NormalizeDouble(CalcHigh - order_step * Point + spread, Digits) > Ask + stop_level * Point))
            {

               price = NormalizeDouble(CalcHigh - order_step * Point + spread, Digits);

               if(stop_loss == 0) sl = 0;
               else sl = NormalizeDouble(CalcHigh - order_step * Point - MathMax(stop_loss, stop_level) * Point, Digits);

               if(take_profit == 0) tp = 0;
               else tp = NormalizeDouble(CalcHigh - order_step * Point + MathMax(take_profit, stop_level) * Point, Digits);
               
               if (OrderModify(order_ticket, price, sl, tp, 0))
                  if(PRINT_REPORT)Print("BuyStop # ",order_ticket);
               else
                  if(PRINT_REPORT)Print("BuyStop # ",order_ticket);

               Sleep(2000);
               RefreshRates(); 
            }
               
         } 

         else if(order_type == OP_SELLSTOP)
           {
            sell_open = false;
            
           
            if(( NormalizeDouble(CalcLow + order_step * Point, Digits) > order_open_price + order_step * Point)&& 
               ( NormalizeDouble(CalcLow + order_step * Point, Digits) < Bid - stop_level * Point))
              {
               price = NormalizeDouble(CalcLow + order_step * Point, Digits);

               if(stop_loss == 0) sl = 0;
               else sl = NormalizeDouble(price + MathMax(stop_loss * Point, stop_level * Point) + spread, Digits);

               if(take_profit == 0) tp = 0;
               else tp = NormalizeDouble(price - MathMax(take_profit * Point, stop_level * Point) + spread, Digits);

               if (OrderModify(OrderTicket(), price, sl, tp, 0))
                  if(PRINT_REPORT)Print("SellStop # ",order_ticket);
               else
                  if(PRINT_REPORT)Print("SellStop # ",GetLastError());
              
               Sleep(2000);
               RefreshRates(); 

              }

           }   

        }

     }
  }

void OpenOrderBuyStop()

{
   lot = Calculate_Lot(LOT, LOT_IS_FIXED, OP_BUY);

   price = NormalizeDouble(CalcHigh - order_step * Point, Digits);

   
   if(NormalizeDouble(Ask + stop_level * Point, Digits) > price) return(0);
  
   
   if(stop_loss == 0) sl = 0;
   else sl = NormalizeDouble(price - MathMax(stop_loss, stop_level) * Point - spread, Digits);
   
   
   if(take_profit == 0) tp = 0;
   else tp = NormalizeDouble(price + MathMax(take_profit, stop_level) * Point - spread, Digits);

   if(OrderSend(Symbol(), OP_BUYSTOP, lot, NormalizeDouble(price,4), 0, NormalizeDouble(sl,4), NormalizeDouble(tp,4), DoubleToStr(MAGIC_NUMBER,0), MAGIC_NUMBER, 0, Blue) >= 0)
   {
      buy_open = false;
      if(PRINT_REPORT)Print("BuyStop #" + DoubleToStr(OrderTicket(),0));
   }
   else 
   {
      if(PRINT_REPORT)Print("BuyStop:" + GetLastError());
   }
   
   Sleep(2000);
   RefreshRates();

}

void OpenOrderSellStop()

  {
   lot = Calculate_Lot(LOT, LOT_IS_FIXED, OP_SELL); 

   price = NormalizeDouble(CalcLow + order_step * Point, Digits);
   
   
   if(NormalizeDouble(Bid - stop_level * Point, Digits) < price) return(0);
   
   
   if(stop_loss == 0) sl = 0;
   else sl = NormalizeDouble(price + MathMax(stop_loss, stop_level)* Point + spread, Digits);
   
   
   if(take_profit == 0) tp = 0;
   else tp = NormalizeDouble(price - MathMax(take_profit, stop_level) * Point + spread, Digits);

   if(OrderSend( Symbol(), OP_SELLSTOP, lot, NormalizeDouble(price,4), 0, NormalizeDouble(sl,4), NormalizeDouble(tp,4), DoubleToStr(MAGIC_NUMBER,0),MAGIC_NUMBER, 0, Blue)>=0)
   {
      sell_open = false;
      if(PRINT_REPORT)Print("SellStop #" + DoubleToStr(OrderTicket(),0));
   }
   else 
   {
      if(PRINT_REPORT)Print("SellStop:" + GetLastError());
   }
   
   Sleep(2000);
   RefreshRates();
}

double Calculate_Lot(double lot_value, bool lot_is_fixed, int direction)

  {
   double acc_free_margin = AccountFreeMargin();
   double calc_margin;
   double price = 0;

   if(direction == OP_BUY)  price = Ask;
   if(direction == OP_SELL) price = Bid;

   if(lot_is_fixed)
   {
      if(LOT_CORRECTION)
      {
         OrderCalcMargin(Symbol(), lot_value, price, calc_margin);
         
         
         if(acc_free_margin < calc_margin)
         {
            lot_value = lot_value * acc_free_margin * 0.9 / calc_margin;
            if(PRINT_REPORT)Print("",lot);
         }
      }

   }
   else
   {
         
         OrderCalcMargin(Symbol(), 1, price, calc_margin);
         lot_value = acc_free_margin * LOT / calc_margin;
   }

   return(NormalizeLot(lot_value));
  }


double NormalizeLot(double lot_value)

{
   double lot_min  = MarketInfo(Symbol(), MODE_MINLOT);
   double lot_max  = MarketInfo(Symbol(), MODE_MAXLOT);
   double lot_step = MarketInfo(Symbol(), MODE_LOTSTEP);

   if(lot_value <= lot_min ) lot_value = lot_min;                 
   else  if(lot_value >= lot_max ) lot_value = lot_max;           
   else lot_value  = MathFloor(lot_value / lot_step) * lot_step;  

   return (NormalizeDouble(lot_value,2)); 
}
 
bool  OrderCalcMargin(string    symbol,      
                      double    volume,      
                      double    price,       
                      double&   margin)      

{
   string first    = StringSubstr(symbol,0,3);
   string second   = StringSubstr(symbol,3,3);
   string currency = AccountCurrency();        
   double leverage = AccountLeverage();        

   double contract = MarketInfo(symbol, MODE_LOTSIZE);
   margin = 0.0;

  
   if(StringLen(symbol) != 6)
     {
      Print("",symbol,"");
      return(false);
     }
 
   if( price <= 0.0 || contract <= 0) 
     {
      Print("",symbol);
      return(false);
     }

   if(first == currency)
   {  
      margin=contract * volume / leverage;
      return(true);
   }

   if((second == currency))
   {
      margin=contract * price * volume / leverage; 
      return(true);
   }

   Print("",symbol);
   return(false);
}